/*
* 07/08/2003 - 19:09:03
*
* XPathNodeAccessHTMLRenderer.java - 
* Copyright (C) 2003 Ashwin Jayaprakash
* ashwinjay@rediffmail.com
* www.javaforu.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package searchassist.ui;

import java.awt.Cursor;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.text.html.HTMLEditorKit;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import netscape.javascript.JSObject;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * @author Ashwin Jayaprakash
 */
public class XPathNodeAccessHTMLRenderer implements IDRenderer
{
    public static final String PARAM_XML_WITH_IDS_FILE = "XMLWithIdsFile";

    private Document doc;
    private javax.swing.JScrollPane rightEditorScrlPane;
    private javax.swing.JEditorPane editorPane;
    private LinkClickListener clickListener;

    private JSObject window;
    private Cursor cursor;
    private Cursor waitCursor;

    private static final boolean DEBUG = true;

    public XPathNodeAccessHTMLRenderer()
    {
    } //end constr

    public JComponent getRenderingComponent()
    {
        return rightEditorScrlPane;
    }

    /**
     * Inits the custom renderer.
     */
    public void init(Attributes attrs)
    {
        try
        {
            // Create a builder factory
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setValidating(false);

            DocumentBuilder docBuilder = factory.newDocumentBuilder();
            // Create the builder and parse the file
            final String xmlIDsFile = attrs.getParameter(PARAM_XML_WITH_IDS_FILE);
            Document inDoc = docBuilder.parse(getClass().getResourceAsStream(xmlIDsFile));

            doc = inDoc;
        }
        catch (SAXParseException spe)
        {
            // Error generated by the parser
            System.out.println("\n** Parsing error" + ", line " + spe.getLineNumber() + ", uri " + spe.getSystemId());
            System.out.println("   " + spe.getMessage());
            // Use the contained exception, if any
            Exception x = spe;
            if (spe.getException() != null)
            {
                x = spe.getException();
            } //end if
            x.printStackTrace();
        }
        catch (SAXException sxe)
        {
            // Error generated by this application
            // (or a parser-initialization error)
            Exception x = sxe;
            if (sxe.getException() != null)
            {
                x = sxe.getException();
            } //end if
            x.printStackTrace();
        }
        catch (ParserConfigurationException pce)
        { // Parser with specified options can't be built
            pce.printStackTrace();
        }
        catch (IOException ioe)
        { // I/O error
            ioe.printStackTrace();
        }

        postInit(attrs);
    } //end method

    /**
     * Method postInit.
     */
    protected void postInit(Attributes attrs)
    {
        rightEditorScrlPane = new javax.swing.JScrollPane();
        rightEditorScrlPane.setBorder(null);

        editorPane = new javax.swing.JEditorPane();
        editorPane.setBorder(null);
        editorPane.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.LOWERED));
        HTMLEditorKit htmlKit = new HTMLEditorKit();
        editorPane.setEditorKit(htmlKit);
        editorPane.setEditable(false);
        editorPane.setFont(new java.awt.Font("Arial", 0, 12));
        rightEditorScrlPane.setViewportView(editorPane);

        clickListener = new LinkClickListener(attrs);
        editorPane.addHyperlinkListener(clickListener);
        
        window = attrs.getWindow();
        // Load some cursors
        cursor = editorPane.getCursor();
        waitCursor = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
    } //end method

    protected void preRenderNodes(List ids, StringBuffer strBuf)
    {
        editorPane.setCursor(waitCursor);
        window.call("waitCursor", null);
        
        strBuf.append("<html><head><base target='_blank'></head><body>\n");
    } //end method

    public void renderNodes(List ids)
    {
        StringBuffer stringBuffer = new StringBuffer();
        preRenderNodes(ids, stringBuffer);

        for (Iterator iter = ids.iterator(); iter.hasNext();)
        {
            String id = (String) iter.next();

            try
            {
                String part = "[@uniqueID='" + id + "']";
                renderNode("//category" + part + " | //item" + part, stringBuffer);
            } //end try
            catch (TransformerException e)
            {
                e.printStackTrace();
            } //end catch
        } //end for

        postRenderNodes(ids, stringBuffer);
    } //end method

    protected void postRenderNodes(List ids, StringBuffer strBuf)
    {
        strBuf.append("</body></html>");
        editorPane.setText(strBuf.toString());
        
        editorPane.setCursor(cursor);
        window.call("defaultCursor", null);
    } //end method

    protected void renderNode(String xpath, StringBuffer stringBuffer) throws TransformerException
    {
        Element parent;
        Element currElement;

        // Get the matching elements
        NodeList nodelist = org.apache.xpath.XPathAPI.selectNodeList(doc, xpath);

        // Process the elements in the nodelist
        for (int i = 0; i < nodelist.getLength(); i++)
        {
            // Get element
            currElement = (Element) nodelist.item(i);

            elementToHTML(currElement, stringBuffer);
            stringBuffer.append("\n<hr>\n");
        } //end for
    } //end method

    protected void elementToHTML(Element element, StringBuffer strBuffer)
    {
        if (element.getNodeName().equalsIgnoreCase("category"))
        {
            strBuffer.append("<li><b>");

            NodeList titleNodes = element.getElementsByTagName("title");
            // Get the text node
            Text text1 = (Text) titleNodes.item(0).getFirstChild();
            String string = text1.getData();
            strBuffer.append(string);

            strBuffer.append("</b></li>\n<ul>");
            NodeList children = element.getChildNodes();
            final int k = children.getLength();
            for (int i = 0; i < k; i++)
            {
                if (children.item(i) instanceof Element)
                {
                    elementToHTML((Element) children.item(i), strBuffer);
                } //end if
            } //end for
            strBuffer.append("</ul>\n");
        } //end if
        else if (element.getNodeName().equalsIgnoreCase("item"))
        {
            strBuffer.append("<li>");

            NodeList linkNodes = element.getElementsByTagName("link");
            // Get the text node
            Text text1 = (Text) linkNodes.item(0).getFirstChild();
            String link = text1.getData().trim();

            NodeList titleNodes = element.getElementsByTagName("title");
            // Get the text node
            text1 = (Text) titleNodes.item(0).getFirstChild();
            String string = text1.getData();
            strBuffer.append("<a href='");
            strBuffer.append(link);
            strBuffer.append("'>");
            strBuffer.append(string);
            strBuffer.append("</a>");

            strBuffer.append("</li>");
        } //end else if
    } //end method
} //end class
