/*
* 07/08/2003 - 19:09:03
*
* KeywordTagAdder.java - 
* Original source: http://java.sun.com/xml/jaxp/dist/1.1/docs/tutorial/sax/work/Echo12.java
* Copyright (C) 2003 Ashwin Jayaprakash
* ashwinjay@rediffmail.com
* www.javaforu.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package searchassist.utils;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.ext.LexicalHandler;
import org.xml.sax.helpers.DefaultHandler;
import com.wallyflint.util.TernarySearchTree;

/**
 * @author Ashwin Jayaprakash
 */
public class KeywordTagAdder extends DefaultHandler implements LexicalHandler
{
	private TernarySearchTree stopWords;
	private String lineEnd = System.getProperty("line.separator");
	private Writer out;
	private String indentString = "   "; // Amount to indent
	private int indentLevel = 0;
	private String currTagName;
	private String keywords;

	private static final List DUMMY_LIST = new LinkedList();
	private static final boolean DEBUG = true;

	public KeywordTagAdder(Writer outWriter, TernarySearchTree tst)
	{
		out = outWriter;
		stopWords = tst;
	} //end constr

	//===========================================================
	// SAX DocumentHandler methods
	//===========================================================

	public void setDocumentLocator(Locator l)
	{
		// Save this to resolve relative URIs or to give diagnostics.
		try
		{
			out.flush();
		}
		catch (IOException e)
		{
			// Ignore errors
		}
	}

	public void startDocument() throws SAXException
	{
		emit("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		nl();
	}

	public void startElement(String namespaceURI, String lName, // local name
	String qName, // qualified name
	Attributes attrs) throws SAXException
	{
		indentLevel++;
		nl();
		String eName = lName; // element name
		if ("".equals(eName))
		{
			eName = qName; // namespaceAware = false
		} //end if

		currTagName = eName;

		emit("<" + eName);
		if (attrs != null)
		{
			for (int i = 0; i < attrs.getLength(); i++)
			{
				String aName = attrs.getLocalName(i); // Attr name 
				if ("".equals(aName))
				{
					aName = attrs.getQName(i);
				} //end if
				emit("\t");
				emit(aName);
				emit("=\"");
				emit(attrs.getValue(i));
				emit("\"");
			} //end for
		} //end if

		emit(">");
	}

	public void characters(char buf[], int offset, int len) throws SAXException
	{
		String s = new String(buf, offset, len).trim();
		if (!s.equals(""))
		{
			//			nl();
			emit(s);

			keywords = null;
			if (currTagName.equals(getTagNameContainingKeywords()))
			{
				keywords = s;
			} //end inner if
		} //end if
	}

	public void ignorableWhitespace(char buf[], int offset, int len) throws SAXException
	{ // Ignore it
	}

	public void processingInstruction(String target, String data) throws SAXException
	{
		nl();
		emit("<?" + target + " " + data + "?>");
	}

	public void endElement(String namespaceURI, String sName, // simple name
	String qName // qualified name
	) throws SAXException
	{
		String eName = sName; // element name
		if ("".equals(eName))
		{
			eName = qName;
			// namespaceAware = false
		} //end if
		nl();
		emit("</" + eName + ">");

		//        if (currTagName.equals(getTagNameContainingKeywords())) causes problems
		if (eName.equals(getTagNameContainingKeywords()))
		{
			nl();
			emit("<" + getTagNameToPutKeywordsIn() + ">");
			nl();
			emit(convertToKeywords(keywords));
			nl();
			emit("</" + getTagNameToPutKeywordsIn() + ">");
		} //end if

		indentLevel--;
	}

	public void endDocument() throws SAXException
	{
		try
		{
			nl();
			out.flush();
		}
		catch (IOException e)
		{
			throw new SAXException("I/O error", e);
		}
	}

	//===========================================================
	// SAX ErrorHandler methods
	//===========================================================
	// treat validation errors as fatal
	public void error(SAXParseException e) throws SAXParseException
	{
		throw e;
	} // dump warnings too
	public void warning(SAXParseException err) throws SAXParseException
	{
		System.out.println("** Warning" + ", line " + err.getLineNumber() + ", uri " + err.getSystemId());
		System.out.println("   " + err.getMessage());
	} //===========================================================
	// LexicalEventListener methods
	//===========================================================
	public void comment(char[] ch, int start, int length) throws SAXException
	{
	}

	public void startCDATA() throws SAXException
	{

	}

	public void endCDATA() throws SAXException
	{

	}

	public void startEntity(java.lang.String name) throws SAXException
	{

	}

	public void endEntity(java.lang.String name) throws SAXException
	{

	}

	public void startDTD(String name, String publicId, String systemId) throws SAXException
	{
	}

	public void endDTD() throws SAXException
	{

	}

	protected String getTagNameContainingKeywords()
	{
		return "title";
	} //end method

	protected String getTagNameToPutKeywordsIn()
	{
		return "keywords";
	} //end method

	//Remove stopwords and separate keywords by comma.
	protected String convertToKeywords(String text)
	{
		if (text == null)
		{
			return "";
		} //end if

		StringTokenizer tokenizer = new StringTokenizer(text);
		String token;
		LinkedList keywords = new LinkedList();
		StringBuffer strBuffer = new StringBuffer();

		while (tokenizer.hasMoreTokens())
		{
			token = tokenizer.nextToken();
			// remove punctuation and convert to lower case
			token = removePunc(token.toLowerCase());

			// check for stop words!
			if (stopWords.get(token) == null && !token.equals(""))
			{
				keywords.add(token);
			} //end if
		} // end while

		for (Iterator iter = keywords.iterator(); iter.hasNext();)
		{
			String element = (String) iter.next();
			strBuffer.append(element);
			if (iter.hasNext())
			{
				strBuffer.append(",");
			} //end if
		} //end for

		return strBuffer.toString();
	} //end method

	//----------------------------------------------------------------
	// Method to remove the punctuation from the word
	//----------------------------------------------------------------
	public static String removePunc(String x)
	{
		String word = "";
		//Punctuations. Not being filterred are: "."
		StringTokenizer tokenizer = new StringTokenizer(x, "()\"'?<>[]~!@#$%^&*+_;:,=|-`");
		while (tokenizer.hasMoreTokens())
		{
			word += tokenizer.nextToken();
		} //end while

		return word;
	} // removePunc

	//===========================================================
	// Utility Methods ...
	//===========================================================
	// Wrap I/O exceptions in SAX exceptions, to
	// suit handler signature requirements
	private void emit(String s) throws SAXException
	{
		try
		{
			out.write(s);
			out.flush();
		}
		catch (IOException e)
		{
			throw new SAXException("I/O error", e);
		}
	} // Start a new line
	// and indent the next line appropriately
	private void nl() throws SAXException
	{
		try
		{
			out.write(lineEnd);
			for (int i = 0; i < indentLevel; i++)
				out.write(indentString);
		}
		catch (IOException e)
		{
			throw new SAXException("I/O error", e);
		}
	}

	public static void main(String argv[])
	{
		if (argv.length < 3)
		{
			System.err.println("Usage: cmd input_xml_filename output_xml_filename stop_words_filename");
			System.exit(1);
		}

		try
		{
			TernarySearchTree tst = new TernarySearchTree();
			BufferedReader stopText = new BufferedReader(new InputStreamReader(new FileInputStream(new File(argv[2]))));
			String stopLine;

			stopLine = stopText.readLine(); // read a line from the stop word list
			// load stop words
			while (stopLine != null) // fill the array with stop words
			{
				tst.put(stopLine, DUMMY_LIST);
				stopLine = stopText.readLine();
			} //end while

			FileOutputStream fos = new FileOutputStream(new File(argv[1]));
			// Set up output stream
			//                        Writer pw = new OutputStreamWriter(System.out, "UTF8");                      
			Writer pw = new OutputStreamWriter(new BufferedOutputStream(fos), "UTF8");
			// Use an instance of ourselves as the SAX event handler
			DefaultHandler handler = new KeywordTagAdder(pw, tst);
			// Use the validating parser
			SAXParserFactory factory = SAXParserFactory.newInstance();
			factory.setValidating(false);
			//                        factory.setNamespaceAware(true);
			// Parse the input
			SAXParser saxParser = factory.newSAXParser();
			XMLReader xmlReader = saxParser.getXMLReader();
			xmlReader.setProperty("http://xml.org/sax/properties/lexical-handler", handler);
			saxParser.parse(new File(argv[0]), handler);
		}
		catch (SAXParseException spe)
		{
			// Error generated by the parser
			System.out.println("\n** Parsing error" + ", line " + spe.getLineNumber() + ", uri " + spe.getSystemId());
			System.out.println("   " + spe.getMessage());
			// Use the contained exception, if any
			Exception x = spe;
			if (spe.getException() != null)
			{
				x = spe.getException();
			} //end if
			x.printStackTrace();
		}
		catch (SAXException sxe)
		{
			// Error generated by this application
			// (or a parser-initialization error)
			Exception x = sxe;
			if (sxe.getException() != null)
			{
				x = sxe.getException();
			} //end if
			x.printStackTrace();
		}
		catch (ParserConfigurationException pce)
		{ // Parser with specified options can't be built
			pce.printStackTrace();
		}
		catch (IOException ioe)
		{ // I/O error
			ioe.printStackTrace();
		}

		System.exit(0);
	} //end main
} //end class
